/**
 * Authors: Frank Benoit <keinfarbton@googlemail.com>
 */
module dwt.dwthelper.FileInputStream;

import dwt.dwthelper.utils;
import dwt.dwthelper.File;
import dwt.dwthelper.InputStream;

version(TANGOSVN){
    import tango.io.device.File;
} else {
    import tango.io.FileConduit;
}
import tango.io.protocol.Reader;
import tango.core.Exception;
import tango.text.convert.Format;

public class FileInputStream : dwt.dwthelper.InputStream.InputStream {

    alias dwt.dwthelper.InputStream.InputStream.read read;

    version(TANGOSVN)
        private tango.io.device.File.File file_;
    else
        private FileConduit conduit;
        
    private ubyte[] buffer;
    private int buf_pos;
    private int buf_size;
    private const int BUFFER_SIZE = 0x10000;
    private bool eof;

    public this ( String name ){
        version(TANGOSVN)
            file_ = new tango.io.device.File.File( name );
        else
            conduit = new FileConduit( name );
            
        buffer = new ubyte[]( BUFFER_SIZE );
    }

    public this ( dwt.dwthelper.File.File file ){
        implMissing( __FILE__, __LINE__ );
        version(TANGOSVN)
            file_ = new tango.io.device.File.File ( file.getAbsolutePath(), tango.io.device.File.File.ReadExisting );
        else
            conduit = new FileConduit( file.getAbsolutePath(), FileConduit.ReadExisting );
        buffer = new ubyte[]( BUFFER_SIZE );
    }

    public override int read(){
        if( eof ){
            return -1;
        }
        try{
            if( buf_pos == buf_size ){
                buf_pos = 0;
                version(TANGOSVN)
                    buf_size = file_.read( buffer );
                else
                    buf_size = conduit.input.read( buffer );
            }
            if( buf_size <= 0 ){
                eof = true;
                return -1;
            }
            assert( buf_pos < BUFFER_SIZE, Format( "{0} {1}", buf_pos, buf_size ) );
            assert( buf_size <= BUFFER_SIZE );
            int res = cast(int) buffer[ buf_pos ];
            buf_pos++;
            return res;
        }
        catch( IOException e ){
            eof = true;
            return -1;
        }
    }

    public long skip( long n ){
        implMissing( __FILE__, __LINE__ );
        return 0L;
    }

    public int available(){
        implMissing( __FILE__, __LINE__ );
        return 0;
    }

    public override void close(){
        version(TANGOSVN)
            file_.close();
        else
            conduit.close();
    }
}


